/*
 *	$Id: config.cpp,v 1.1.1.1 2001/10/30 00:30:11 fault Exp $
 *
 */

#include "config.h"
#include <qdir.h>
#include <qregexp.h>
#include <qwhatsthis.h>
#include <kglobal.h>
#include <klocale.h>
#include <kstddirs.h>
#include <kdirwatch.h>
#include <kapp.h>
#include <qlayout.h>
#include <kcombobox.h>

extern "C"
{
	QObject *allocate_config (KConfig * conf, QWidget * parent)
	{
		return (new KBoxConfig (conf, parent));
	}
}

KBoxConfig::KBoxConfig (KConfig * conf, QWidget * parent):QObject (parent)
{
	KBoxConfiga = new KConfig ("kwinkboxrc");
	KGlobal::locale ()->insertCatalogue ("libkwinkbox_config");

	gb1 = new QGroupBox (1, Qt::Horizontal,
		i18n ("blackbox style Selector"), parent);

	styleListBox = new QListBox (gb1);
	QWhatsThis::add (styleListBox,
		i18n ("Make your blackbox selection by clicking on a style here. "));

	styleLabel =
		new QLabel (i18n ("To manage your blackbox styles, simply click on the "
			"link below to open a Konqueror window. Once shown, you "
			"will be able to add or remove native blackbox styles, by "
			"uncompressing <b>http://blackbox.themes.org/</b> style files "
			"into this directory, or creating symlinks to "
			"existing blackbox styles on your system."), parent);

	urlLabel = new KURLLabel (parent);
	urlLabel->
		setText (i18n
		("Open Konqueror Window at KDE's blackbox style directory"));

	gb2 = new QGroupBox (1, Qt::Horizontal,
		i18n ("Title Settings"), parent);

	cbstyleTitleTextColors =
		new QCheckBox (i18n ("Use style &title text colors"), gb2);

	QWhatsThis::add (cbstyleTitleTextColors,
		i18n ("When selected, titlebar colors will follow those set "
			"in the blackbox style. If not selected, the current KDE "
			"titlebar colors will be used instead."));

	cbstyleTitleFont =
		new QCheckBox (i18n ("&Use font specified in style"), gb2);

	QWhatsThis::add (cbstyleTitleFont,
		i18n ("When selected, the font for the titlebar will follow those set"
		"in the blackbox style. If not selected, the current KDE "
		"font selection will be used instead."));

	QHBoxLayout *hb=new QHBoxLayout(gb2);
	justLabel = new QLabel(gb2);
	justLabel->setText( i18n( "Title Justification:" ) );
	hb->addWidget(justLabel);

	justBox = new KComboBox( FALSE, gb2);
	    justBox->insertItem( i18n( "Style Default" ) );
		    justBox->insertItem( i18n( "Left" ) );
			    justBox->insertItem( i18n( "Center" ) );
				    justBox->insertItem( i18n( "Right" ) );
					    justBox->setEditable( FALSE );
	hb->addWidget(justBox);
	
	// Ensure we track user changes properly
	connect (styleListBox, SIGNAL (selectionChanged ()),
		this, SLOT (slotSelectionChanged ()));

	connect (urlLabel, SIGNAL (leftClickedURL (const QString &)),
		this, SLOT (callURL (const QString &)));

	connect (cbstyleTitleTextColors, SIGNAL (clicked ()),
		this, SLOT (slotSelectionChanged ()));

	connect (cbstyleTitleFont, SIGNAL (clicked ()),
		this, SLOT (slotSelectionChanged ()));

	connect (justBox, SIGNAL(activated (int)),this, SLOT(justSelectionChanged(int)));
	
	// Create the style directory (if not found) ... and obtain the path as we do so.
	localstylestring = KGlobal::dirs ()->saveLocation ("data", "kwin");
	localstylestring += "/blackbox-styles";

	if (!QFile::exists (localstylestring))
		QDir ().mkdir (localstylestring);

	// Load configuration options
	load (conf);

	// Watch the blackbox style directory for style additions/removals
	KDirWatch::self ()->addDir (localstylestring);
	connect (KDirWatch::self (), SIGNAL (dirty (const QString &)), this,
		SLOT (findblackboxstyles ()));

	// Set the konqui link url
	QString urlstylestring = QString ("file://") + localstylestring;
	urlstylestring.replace (QRegExp ("~"), "$HOME");
	urlLabel->setURL (urlstylestring);

	// Make the widgets visible in kwindecoration
	gb1->show ();
	styleLabel->show ();
	urlLabel->show ();
	gb2->show ();

	KDirWatch::self ()->startScan ();
}

KBoxConfig::~KBoxConfig ()
{
	KDirWatch::self ()->removeDir (localstylestring);
	KDirWatch::self ()->stopScan ();
	delete gb2;
	delete urlLabel;
	delete styleLabel;
	delete gb1;
	delete KBoxConfiga;
}

// Searches for all installed blackbox styles, and adds them to the listBox.
void KBoxConfig::findblackboxstyles ()
{
	styleListBox->clear ();
	styleListBox->insertItem (i18n ("default"));
	QDir d (localstylestring, QString ("*"), QDir::Name,
		QDir::Files | QDir::Readable);
	if (d.exists ())
	{
		QFileInfoListIterator it2 (*d.entryInfoList ());
		QFileInfo *finfo;
		while ((finfo = it2.current ()))
		{
			if ( !styleListBox->findItem( finfo->fileName()) )
			styleListBox->insertItem (finfo->fileName ());
			++it2;
		}
	}

	// Select the currently used blackbox style
	QString styleName = KBoxConfiga->readEntry ("Currentstyle", "");

	// Provide a style alias
//	if (styleName == "default")
//		styleName = "";

	//if (styleName == "")
	//	styleListBox->setCurrentItem (styleListBox->findItem (i18n ("Nyz")));
	//else
		styleListBox->setCurrentItem (styleListBox->findItem (styleName));
}

void KBoxConfig::callURL (const QString & s)
{
	kapp->invokeBrowser (s);
}

void KBoxConfig::slotSelectionChanged ()
{
	emit changed ();
}

void KBoxConfig::justSelectionChanged (int)
{
	    emit changed ();
}

// Loads the configurable options from the kwinblackboxrc config file
void KBoxConfig::load (KConfig *)
{
	KBoxConfiga->setGroup ("General");

	bool override = KBoxConfiga->readBoolEntry ("styleTitleTextColors", true);
	cbstyleTitleTextColors->setChecked (override);

	override = KBoxConfiga->readBoolEntry ("styleTitleFont", true);
	cbstyleTitleFont->setChecked (override);

	int i = KBoxConfiga->readNumEntry ("styleTitleJust",0);
	justBox->setCurrentItem(i);

	findblackboxstyles ();
}

// Saves the configurable options to the kwinblackboxrc config file
void KBoxConfig::save (KConfig *)
{
	KBoxConfiga->setGroup ("General");
	KBoxConfiga->writeEntry ("styleTitleTextColors",
		cbstyleTitleTextColors->isChecked ());
	KBoxConfiga->writeEntry ("styleTitleFont", cbstyleTitleFont->isChecked ());
KBoxConfiga->writeEntry ("styleTitleJust", justBox->currentItem());
	if (styleListBox->currentText () == "Nyz")
		KBoxConfiga->writeEntry ("Currentstyle", "default");
	else
		KBoxConfiga->writeEntry ("Currentstyle", styleListBox->currentText ());

	KBoxConfiga->sync ();
}

// Sets UI widget defaults which must correspond to config defaults
void KBoxConfig::defaults ()
{
	cbstyleTitleTextColors->setChecked (true);
	cbstyleTitleFont->setChecked (true);
	styleListBox->setCurrentItem (styleListBox->findItem (i18n ("Nyz")));
}

#include "config.moc"
// vim: ts=4
